@@ -272,10 +272,7 @@ impl PrefixMap {
                 Some(color) => ":".color(color),
                 None => ColoredString::from(":"),
             };
-            Some(format!(
-                "{}{}{}",
-                prefix_colored, semicolon_colored, rest_colored
-            ))
+            Some(format!("{prefix_colored}{semicolon_colored}{rest_colored}"))
         } else {
             None
         };
@@ -334,7 +331,7 @@ impl PrefixMap {
             };
             let length = prefix_colored.len() + 1 + rest_colored.len();
             (
-                format!("{}{}{}", prefix_colored, semicolon_colored, rest_colored),
+                format!("{prefix_colored}{semicolon_colored}{rest_colored}"),
                 length,
             )
         } else {
