@@ -83,9 +83,12 @@ pub enum Command {
         )]
         result_schema_format: ShExFormat,
 
-        #[arg(short = 't', long = "show elapsed time")]
+        #[arg(short = 't', long = "show-time")]
         show_time: Option<bool>,
 
+        #[arg(long = "show-schema")]
+        show_schema: Option<bool>,
+
         #[arg(long = "statistics")]
         show_statistics: Option<bool>,
 
@@ -104,6 +107,18 @@ pub enum Command {
         )]
         reader_mode: RDFReaderMode,
 
+        #[arg(
+            long = "show-dependencies",
+            value_name = "Show dependencies between shapes"
+        )]
+        show_dependencies: Option<bool>,
+
+        #[arg(
+            long = "compile",
+            value_name = "Compile Schema to Internal representation"
+        )]
+        compile: Option<bool>,
+
         #[arg(
             long = "force-overwrite",
             value_name = "Force overwrite mode",
@@ -191,6 +206,14 @@ pub enum Command {
         )]
         reader_mode: RDFReaderMode,
 
+        #[arg(
+            short = 'r',
+            long = "result-format",
+            value_name = "Ouput result format",
+            default_value_t = ResultFormat::Compact
+        )]
+        result_format: ResultFormat,
+
         #[arg(
             short = 'o',
             long = "output-file",
@@ -266,6 +289,14 @@ pub enum Command {
         #[arg(short = 'e', long = "endpoint", value_name = "Endpoint with RDF data")]
         endpoint: Option<String>,
 
+        #[arg(
+            short = 'r',
+            long = "result-format",
+            value_name = "Ouput result format",
+            default_value_t = ResultFormat::Turtle
+        )]
+        result_format: ResultFormat,
+
         #[arg(
             short = 'o',
             long = "output-file",
@@ -330,6 +361,14 @@ pub enum Command {
         )]
         mode: ShaclValidationMode,
 
+        #[arg(
+            short = 'r',
+            long = "result-format",
+            value_name = "Ouput result format",
+            default_value_t = ResultFormat::Compact
+        )]
+        result_format: ResultFormat,
+
         #[arg(
             short = 'o',
             long = "output-file",
@@ -373,13 +412,6 @@ pub enum Command {
         )]
         reader_mode: RDFReaderMode,
 
-        #[arg(
-            short = 'o',
-            long = "output-file",
-            value_name = "Output file name, default = terminal"
-        )]
-        output: Option<PathBuf>,
-
         #[arg(
             short = 'r',
             long = "result-format",
@@ -388,6 +420,13 @@ pub enum Command {
         )]
         result_format: DataFormat,
 
+        #[arg(
+            short = 'o',
+            long = "output-file",
+            value_name = "Output file name, default = terminal"
+        )]
+        output: Option<PathBuf>,
+
         /// Config file path, if unset it assumes default config
         #[arg(short = 'c', long = "config-file", value_name = "Config file name")]
         config: Option<PathBuf>,
@@ -821,6 +860,34 @@ pub enum DataFormat {
     NQuads,
 }
 
+#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, ValueEnum, Debug)]
+#[clap(rename_all = "lower")]
+pub enum ResultFormat {
+    Turtle,
+    NTriples,
+    RDFXML,
+    TriG,
+    N3,
+    NQuads,
+    Compact,
+    Json,
+}
+
+impl Display for ResultFormat {
+    fn fmt(&self, dest: &mut Formatter<'_>) -> Result<(), std::fmt::Error> {
+        match self {
+            ResultFormat::Turtle => write!(dest, "turtle"),
+            ResultFormat::NTriples => write!(dest, "ntriples"),
+            ResultFormat::RDFXML => write!(dest, "rdfxml"),
+            ResultFormat::TriG => write!(dest, "trig"),
+            ResultFormat::N3 => write!(dest, "n3"),
+            ResultFormat::NQuads => write!(dest, "nquads"),
+            ResultFormat::Compact => write!(dest, "compact"),
+            ResultFormat::Json => write!(dest, "json"),
+        }
+    }
+}
+
 pub trait MimeType {
     fn mime_type(&self) -> String;
 }
