@@ -18,8 +18,8 @@ use anyhow::*;
 use clap::Parser;
 use cli::{
     Cli, Command, DCTapFormat, DCTapResultFormat, DataFormat, InputConvertMode, MimeType,
-    OutputConvertMode, RDFReaderMode, ResultQueryFormat, ResultServiceFormat, ShowNodeMode,
-    ValidationMode,
+    OutputConvertMode, RDFReaderMode, ResultFormat, ResultQueryFormat, ResultServiceFormat,
+    ShowNodeMode, ValidationMode,
 };
 use dctap::DCTAPFormat;
 use iri_s::IriS;
@@ -28,13 +28,15 @@ use rudof_lib::{
     Rudof, RudofConfig, ShExFormat, ShExFormatter, ShaclFormat, ShaclValidationMode,
     ShapeMapFormatter, ShapeMapParser, ShapesGraphSource,
 };
-use shapemap::{NodeSelector, ShapeMapFormat as ShapemapFormat, ShapeSelector};
+use shacl_validation::validation_report::report::ValidationReport;
+use shapemap::{NodeSelector, ResultShapeMap, ShapeMapFormat as ShapemapFormat, ShapeSelector};
 use shapes_converter::ShEx2Sparql;
 use shapes_converter::{ImageFormat, ShEx2Html, ShEx2Uml, Shacl2ShEx, Tap2ShEx, UmlGenerationMode};
 use shex_ast::object_value::ObjectValue;
 use shex_ast::{ShapeExprLabel, SimpleReprSchema};
 use sparql_service::{RdfData, ServiceDescription};
-use srdf::{Query, QuerySolution, RDFFormat, ReaderMode, VarName};
+use srdf::NeighsRDF;
+use srdf::{QuerySolution, RDFFormat, ReaderMode, SRDFGraph, VarName};
 use std::collections::HashMap;
 use std::fs::{File, OpenOptions};
 use std::io::{self, BufWriter, Write};
@@ -44,7 +46,6 @@ use std::str::FromStr;
 use std::time::Instant;
 use supports_color::Stream;
 use tracing::debug;
-
 pub mod cli;
 pub mod input_convert_format;
 pub mod input_spec;
@@ -111,17 +112,25 @@ fn main() -> Result<()> {
             schema,
             schema_format,
             result_schema_format,
+            show_dependencies,
             output,
             show_time,
+            show_schema,
             show_statistics,
+            compile,
             force_overwrite,
             reader_mode,
             config,
         }) => {
             let config = get_config(config)?;
-            /*if let Some(flag) = show_statistics {
-                config.set_show_extends(*flag);
-            }*/
+            if let Some(show_dependencies) = show_dependencies {
+                config
+                    .shex_config()
+                    .with_show_dependencies(*show_dependencies);
+            }
+            if let Some(flag) = show_statistics {
+                config.shex_config().set_show_extends(*flag);
+            }
             let show_time = (*show_time).unwrap_or_default();
             run_shex(
                 schema,
@@ -129,6 +138,8 @@ fn main() -> Result<()> {
                 result_schema_format,
                 output,
                 show_time,
+                show_schema.unwrap_or_default(),
+                compile.unwrap_or_default(),
                 *force_overwrite,
                 reader_mode,
                 &config,
@@ -148,6 +159,7 @@ fn main() -> Result<()> {
             shapemap_format,
             max_steps,
             shacl_validation_mode,
+            result_format,
             output,
             config,
             force_overwrite,
@@ -166,6 +178,7 @@ fn main() -> Result<()> {
                     shapemap,
                     shapemap_format,
                     cli.debug,
+                    result_format,
                     output,
                     &config,
                     *force_overwrite,
@@ -187,6 +200,7 @@ fn main() -> Result<()> {
                         reader_mode,
                         *shacl_validation_mode,
                         cli.debug,
+                        result_format,
                         output,
                         &config,
                         *force_overwrite,
@@ -205,6 +219,7 @@ fn main() -> Result<()> {
             shape,
             shapemap,
             shapemap_format,
+            result_format,
             output,
             config,
             force_overwrite,
@@ -222,6 +237,7 @@ fn main() -> Result<()> {
                 shapemap,
                 shapemap_format,
                 cli.debug,
+                result_format,
                 output,
                 &config,
                 *force_overwrite,
@@ -235,6 +251,7 @@ fn main() -> Result<()> {
             reader_mode,
             endpoint,
             mode,
+            result_format,
             output,
             force_overwrite,
             config,
@@ -249,6 +266,7 @@ fn main() -> Result<()> {
                 reader_mode,
                 *mode,
                 cli.debug,
+                result_format,
                 output,
                 &config,
                 *force_overwrite,
@@ -445,6 +463,8 @@ fn run_shex(
     result_schema_format: &CliShExFormat,
     output: &Option<PathBuf>,
     show_time: bool,
+    show_schema: bool,
+    compile: bool,
     force_overwrite: bool,
     _reader_mode: &RDFReaderMode,
     config: &RudofConfig,
@@ -452,8 +472,11 @@ fn run_shex(
     let begin = Instant::now();
     let (writer, color) = get_writer(output, force_overwrite)?;
     let mut rudof = Rudof::new(config);
+
     parse_shex_schema_rudof(&mut rudof, input, schema_format, config)?;
-    show_schema_rudof(&rudof, result_schema_format, writer, color)?;
+    if show_schema {
+        show_schema_rudof(&rudof, result_schema_format, writer, color)?;
+    }
     if show_time {
         let elapsed = begin.elapsed();
         let _ = writeln!(io::stderr(), "elapsed: {:.03?} sec", elapsed.as_secs_f64());
@@ -483,6 +506,26 @@ fn run_shex(
             writeln!(io::stderr(), "{label} from {iri}")?
         }
     }
+    if compile && config.show_ir() {
+        writeln!(io::stdout(), "\nIR:")?;
+        if let Some(shex_ir) = rudof.get_shex_ir() {
+            writeln!(io::stdout(), "ShEx IR:")?;
+            writeln!(io::stdout(), "{shex_ir}")?;
+        } else {
+            bail!("Internal error: No ShEx schema read")
+        }
+    }
+    if compile && config.show_dependencies() {
+        writeln!(io::stdout(), "\nDependencies:")?;
+        if let Some(shex_ir) = rudof.get_shex_ir() {
+            for (source, posneg, target) in shex_ir.dependencies() {
+                writeln!(io::stdout(), "{source}-{posneg}->{target}")?;
+            }
+        } else {
+            bail!("Internal error: No ShEx schema read")
+        }
+        writeln!(io::stdout(), "---end dependencies\n")?;
+    }
     Ok(())
 }
 
@@ -553,13 +596,14 @@ fn run_validate_shex(
     shapemap: &Option<InputSpec>,
     shapemap_format: &CliShapeMapFormat,
     _debug: u8,
+    result_format: &ResultFormat,
     output: &Option<PathBuf>,
     config: &RudofConfig,
     force_overwrite: bool,
 ) -> Result<()> {
     if let Some(schema) = schema {
         let mut rudof = Rudof::new(config);
-        let (mut writer, _color) = get_writer(output, force_overwrite)?;
+        let (writer, _color) = get_writer(output, force_overwrite)?;
         let schema_format = schema_format.unwrap_or_default();
         let schema_reader = schema.open_read(Some(&schema_format.mime_type()))?;
         let schema_format = match schema_format {
@@ -599,13 +643,76 @@ fn run_validate_shex(
             }
         };
         let result = rudof.validate_shex()?;
-        writeln!(writer, "Result:\n{}", result)?;
+        write_result_shapemap(writer, result_format, result)?;
         Ok(())
     } else {
         bail!("No ShEx schema specified")
     }
 }
 
+fn write_validation_report(
+    mut writer: Box<dyn Write + 'static>,
+    format: &ResultFormat,
+    report: ValidationReport,
+) -> Result<()> {
+    match format {
+        ResultFormat::Compact => {
+            writeln!(writer, "Validation report: {report}")?;
+        }
+        ResultFormat::Json => {
+            bail!("Generation of JSON for SHACl validation report is not implemented yet")
+            /*let str = serde_json::to_string_pretty(&report)
+                .context("Error converting Result to JSON: {result}")?;
+            writeln!(writer, "{str}")?;*/
+        }
+        _ => {
+            use crate::srdf::BuildRDF;
+            let mut rdf_writer = SRDFGraph::new();
+            report.to_rdf(&mut rdf_writer)?;
+            let rdf_format = result_format_to_rdf_format(format)?;
+            rdf_writer.serialize(&rdf_format, &mut writer)?;
+        }
+    }
+    Ok(())
+}
+
+fn result_format_to_rdf_format(result_format: &ResultFormat) -> Result<RDFFormat> {
+    match result_format {
+        ResultFormat::Turtle => Ok(RDFFormat::Turtle),
+        ResultFormat::NTriples => Ok(RDFFormat::NTriples),
+        ResultFormat::RDFXML => Ok(RDFFormat::RDFXML),
+        ResultFormat::TriG => Ok(RDFFormat::TriG),
+        ResultFormat::N3 => Ok(RDFFormat::N3),
+        ResultFormat::NQuads => Ok(RDFFormat::NQuads),
+        _ => bail!("Unsupported result format {result_format}"),
+    }
+}
+
+fn write_result_shapemap(
+    mut writer: Box<dyn Write + 'static>,
+    format: &ResultFormat,
+    result: ResultShapeMap,
+) -> Result<()> {
+    match format {
+        ResultFormat::Turtle => todo!(),
+        ResultFormat::NTriples => todo!(),
+        ResultFormat::RDFXML => todo!(),
+        ResultFormat::TriG => todo!(),
+        ResultFormat::N3 => todo!(),
+        ResultFormat::NQuads => todo!(),
+        ResultFormat::Compact => {
+            writeln!(writer, "Result:")?;
+            result.show_minimal(writer)?;
+        }
+        ResultFormat::Json => {
+            let str = serde_json::to_string_pretty(&result)
+                .context("Error converting Result to JSON: {result}")?;
+            writeln!(writer, "{str}")?;
+        }
+    }
+    Ok(())
+}
+
 #[allow(clippy::too_many_arguments)]
 fn run_validate_shacl(
     schema: &Option<InputSpec>,
@@ -616,14 +723,15 @@ fn run_validate_shacl(
     reader_mode: &RDFReaderMode,
     mode: ShaclValidationMode,
     _debug: u8,
+    result_format: &ResultFormat,
     output: &Option<PathBuf>,
     config: &RudofConfig,
     force_overwrite: bool,
 ) -> Result<()> {
-    let (mut writer, _color) = get_writer(output, force_overwrite)?;
+    let (writer, _color) = get_writer(output, force_overwrite)?;
     let mut rudof = Rudof::new(config);
     get_data_rudof(&mut rudof, data, data_format, endpoint, reader_mode, config)?;
-    let result = if let Some(schema) = schema {
+    let validation_report = if let Some(schema) = schema {
         let reader_mode = reader_mode_convert(*reader_mode);
         let shapes_format = shapes_format.unwrap_or_default();
         add_shacl_schema_rudof(&mut rudof, schema, &shapes_format, &reader_mode, config)?;
@@ -632,7 +740,8 @@ fn run_validate_shacl(
         rudof.validate_shacl(&mode, &ShapesGraphSource::current_data())
     }?;
 
-    writeln!(writer, "Result:\n{}", result)?;
+    write_validation_report(writer, result_format, validation_report)?;
+
     Ok(())
 }
 
@@ -948,7 +1057,7 @@ fn run_shex2sparql(
         let converter = ShEx2Sparql::new(&config.shex2sparql_config());
         let sparql = converter.convert(schema, shape)?;
         let (mut writer, _color) = get_writer(output, force_overwrite)?;
-        write!(writer, "{}", sparql)?;
+        write!(writer, "{sparql}")?;
     }
     Ok(())
 }
@@ -1168,7 +1277,7 @@ fn show_node_info<S, W: Write>(
     writer: &mut W,
 ) -> Result<()>
 where
-    S: Query,
+    S: NeighsRDF,
 {
     for node in node_selector.iter_node(rdf) {
         let subject = node_to_subject(node, rdf)?;
@@ -1244,7 +1353,7 @@ where
 
 fn cnv_predicates<S>(predicates: &Vec<String>, rdf: &S) -> Result<Vec<S::IRI>>
 where
-    S: Query,
+    S: NeighsRDF,
 {
     let mut vs = Vec::new();
     for s in predicates {
@@ -1282,7 +1391,7 @@ fn run_shapemap(
 
 fn node_to_subject<S>(node: &ObjectValue, rdf: &S) -> Result<S::Subject>
 where
-    S: Query,
+    S: NeighsRDF,
 {
     match node {
         ObjectValue::IriRef(iri_ref) => {
@@ -1293,8 +1402,8 @@ where
                     iri_s.into()
                 }
             };
-            let term: S::Term = iri.into();
-            match term.clone().try_into() {
+            let term: S::Term = iri.into().into();
+            match S::term_as_subject(&term) {
                 Ok(subject) => Ok(subject),
                 Err(_) => bail!("node_to_subject: Can't convert term {term} to subject"),
             }
@@ -1360,8 +1469,8 @@ fn show_variables<'a, W: Write>(
     vars: impl Iterator<Item = &'a VarName>,
 ) -> Result<()> {
     for var in vars {
-        let str = format!("{}", var);
-        write!(writer, "{:15}", str)?;
+        let str = format!("{var}");
+        write!(writer, "{str:15}")?;
     }
     writeln!(writer)?;
     Ok(())
@@ -1378,15 +1487,15 @@ fn show_result<W: Write>(
                 oxrdf::Term::NamedNode(named_node) => {
                     let (str, _length) =
                         prefixmap.qualify_and_length(&IriS::from_named_node(named_node));
-                    format!("{}     ", str)
+                    format!("{str}     ")
                 }
-                oxrdf::Term::BlankNode(blank_node) => format!("  {}", blank_node),
-                oxrdf::Term::Literal(literal) => format!("  {}", literal),
-                oxrdf::Term::Triple(triple) => format!("  {}", triple),
+                oxrdf::Term::BlankNode(blank_node) => format!("  {blank_node}"),
+                oxrdf::Term::Literal(literal) => format!("  {literal}"),
+                oxrdf::Term::Triple(triple) => format!("  {triple}"),
             },
             None => String::new(),
         };
-        write!(writer, "{:15}", str)?;
+        write!(writer, "{str:15}")?;
     }
     writeln!(writer)?;
     Ok(())
@@ -1405,6 +1514,14 @@ fn parse_shex_schema_rudof(
     let shex_config = config.shex_config();
     let base = base_convert(&shex_config.base);
     rudof.read_shex(reader, &schema_format, base)?;
+    if config.shex_config().check_well_formed() {
+        println!("Checking well formedness...");
+        let shex_ir = rudof.get_shex_ir().unwrap();
+        if shex_ir.has_neg_cycle() {
+            let neg_cycles = shex_ir.neg_cycles();
+            bail!("Schema contains negative cycles: {neg_cycles:?}");
+        }
+    }
     Ok(())
 }
 
@@ -1429,6 +1546,9 @@ fn parse_dctap(rudof: &mut Rudof, input: &InputSpec, format: &DCTapFormat) -> Re
             }
             InputSpec::Stdin => bail!("Can not read Excel file from stdin"),
             InputSpec::Url(_) => bail!("Not implemented reading Excel files from URIs yet"),
+            InputSpec::Str(_) => {
+                bail!("Not implemented reading Excel files from strings yet")
+            }
         },
     }
 }
@@ -1441,9 +1561,7 @@ fn shacl_format_convert(shacl_format: &cli::ShaclFormat) -> Result<ShaclFormat>
         cli::ShaclFormat::TriG => Ok(ShaclFormat::TriG),
         cli::ShaclFormat::N3 => Ok(ShaclFormat::N3),
         cli::ShaclFormat::NQuads => Ok(ShaclFormat::NQuads),
-        cli::ShaclFormat::Internal => {
-            bail!("Cannot convert internal SHACL format to RDF data format")
-        }
+        cli::ShaclFormat::Internal => Ok(ShaclFormat::Internal),
     }
 }
 
