@@ -1,15 +1,14 @@
 use crate::{RudofConfig, RudofError, ShapesGraphSource};
 use iri_s::IriS;
-use shacl_ast::{ShaclParser, ShaclWriter};
+use shacl_rdf::{ShaclParser, ShaclWriter};
 use shacl_validation::shacl_processor::{GraphValidation, ShaclProcessor};
 use shacl_validation::store::graph::Graph;
 
 use shapemap::{NodeSelector, ShapeSelector};
 use shapes_converter::{ShEx2Uml, Tap2ShEx};
-use shex_ast::compiled::compiled_schema::CompiledSchema;
+use shex_ast::ir::schema_ir::SchemaIR;
 use shex_compact::ShExParser;
 use shex_validation::{ResolveMethod, SchemaWithoutImports};
-use srdf::Sparql;
 use srdf::{FocusRDF, SRDFGraph};
 use std::fmt::Debug;
 use std::path::Path;
@@ -27,9 +26,12 @@ pub use shapemap::{QueryShapeMap, ResultShapeMap, ShapeMapFormat, ValidationStat
 pub use shex_compact::{ShExFormatter, ShapeMapParser, ShapemapFormatter as ShapeMapFormatter};
 pub use shex_validation::Validator as ShExValidator;
 pub use shex_validation::{ShExFormat, ValidatorConfig};
+use srdf::QueryRDF;
 pub use srdf::{QuerySolution, QuerySolutions, RDFFormat, ReaderMode, SRDFSparql, VarName};
+
 pub type Result<T> = result::Result<T, RudofError>;
 pub use shacl_ast::ast::Schema as ShaclSchema;
+pub use shacl_ir::compiled::schema::SchemaIR as ShaclSchemaIR;
 pub use shapes_converter::UmlGenerationMode;
 pub use shex_ast::Schema as ShExSchema;
 pub use sparql_service::RdfData;
@@ -39,12 +41,15 @@ pub use sparql_service::RdfData;
 pub struct Rudof {
     config: RudofConfig,
     rdf_data: RdfData,
+    shacl_schema: Option<ShaclSchema<RdfData>>,
+    shacl_schema_ir: Option<ShaclSchemaIR>,
     shex_schema: Option<ShExSchema>,
-    shacl_schema: Option<ShaclSchema>, // TODO: Should we store a compiled schema to avoid compiling it for each validation request?
+    shex_schema_ir: Option<SchemaIR>,
     resolved_shex_schema: Option<SchemaWithoutImports>,
     shex_validator: Option<ShExValidator>,
     shapemap: Option<QueryShapeMap>,
     dctap: Option<DCTAP>,
+    shex_results: Option<ResultShapeMap>,
 }
 
 // TODO: We added this declaration so PyRudof can contain Rudof and be Send as required by PyO3
@@ -56,12 +61,15 @@ impl Rudof {
         Rudof {
             config: config.clone(),
             shex_schema: None,
+            shex_schema_ir: None,
             shacl_schema: None,
+            shacl_schema_ir: None,
             resolved_shex_schema: None,
             shex_validator: None,
             rdf_data: RdfData::new(),
             shapemap: None,
             dctap: None,
+            shex_results: None,
         }
     }
 
@@ -101,20 +109,33 @@ impl Rudof {
     /// Get the shapes graph schema from the current RDF data
     pub fn get_shacl_from_data(&mut self) -> Result<()> {
         let schema = shacl_schema_from_data(self.rdf_data.clone())?;
-        self.shacl_schema = Some(schema);
+        self.shacl_schema = Some(schema.clone());
+        let shacl_ir = ShaclSchemaIR::compile(&schema)
+            .map_err(|e| RudofError::ShaclCompilation { error: Box::new(e) })?;
+        self.shacl_schema_ir = Some(shacl_ir);
         Ok(())
     }
 
     /// Get the current SHACL
-    pub fn get_shacl(&self) -> Option<&ShaclSchema> {
+    pub fn get_shacl(&self) -> Option<&ShaclSchema<RdfData>> {
         self.shacl_schema.as_ref()
     }
 
+    /// Get the current SHACL Schema Internal Representation
+    pub fn get_shacl_ir(&self) -> Option<&ShaclSchemaIR> {
+        self.shacl_schema_ir.as_ref()
+    }
+
     /// Get the current ShEx Schema
     pub fn get_shex(&self) -> Option<&ShExSchema> {
         self.shex_schema.as_ref()
     }
 
+    /// Get the current ShEx Schema Internal Representation
+    pub fn get_shex_ir(&self) -> Option<&SchemaIR> {
+        self.shex_schema_ir.as_ref()
+    }
+
     /// Get the current DCTAP
     pub fn get_dctap(&self) -> Option<&DCTAP> {
         self.dctap.as_ref()
@@ -277,13 +298,13 @@ impl Rudof {
         if let Some(shacl) = &self.shacl_schema {
             match format {
                 ShaclFormat::Internal => {
-                    write!(writer, "{shacl:?}").map_err(|e| RudofError::SerializingSHACLInternal {
+                    write!(writer, "{shacl}").map_err(|e| RudofError::SerializingSHACLInternal {
                         error: format!("{e}"),
                     })
                 }
                 _ => {
                     let data_format = shacl_format2rdf_format(format)?;
-                    let mut shacl_writer: ShaclWriter<SRDFGraph> = ShaclWriter::new();
+                    let mut shacl_writer: ShaclWriter<RdfData> = ShaclWriter::new();
                     shacl_writer
                         .write(shacl)
                         .map_err(|e| RudofError::WritingSHACL {
@@ -308,9 +329,10 @@ impl Rudof {
     /// The action is necessary to start a fresh validation
     pub fn reset_validation_results(&mut self) {
         // TODO: We could add another operation to reset only the current validation results keeping the compiled schema
-        if let Some(ref mut validator) = &mut self.shex_validator {
+        /*if let Some(ref mut validator) = &mut self.shex_validator {
             validator.reset_result_map()
-        }
+        }*/
+        self.shex_results = None
     }
 
     /// Resets the current validator
@@ -346,7 +368,10 @@ impl Rudof {
                     error: format!("{e}"),
                 }
             })?;
-        let schema = shacl_schema_from_data(rdf_graph)?;
+        let rdf_data = RdfData::from_graph(rdf_graph).map_err(|e| RudofError::ReadError {
+            error: format!("Obtaining SHACL from rdf_data: {e}"),
+        })?;
+        let schema = shacl_schema_from_data(rdf_data)?;
         self.shacl_schema = Some(schema);
         Ok(())
     }
@@ -375,28 +400,27 @@ impl Rudof {
     /// - `format` indicates the DCTAP format
     pub fn read_dctap_path<P: AsRef<Path>>(&mut self, path: P, format: &DCTAPFormat) -> Result<()> {
         let path_name = path.as_ref().display().to_string();
-        let dctap =
-            match format {
-                DCTAPFormat::CSV => {
-                    let dctap = DCTAP::from_path(path, &self.config.tap_config()).map_err(|e| {
-                        RudofError::DCTAPReaderCSV {
-                            path: path_name,
-                            error: format!("{e}"),
-                        }
-                    })?;
-                    Ok(dctap)
-                }
-                DCTAPFormat::XLS | DCTAPFormat::XLSB | DCTAPFormat::XLSM | DCTAPFormat::XLSX => {
-                    let path_buf = path.as_ref().to_path_buf();
-                    let dctap = DCTAP::from_excel(path_buf, None, &self.config.tap_config())
-                        .map_err(|e| RudofError::DCTAPReaderPathXLS {
-                            path: path_name,
-                            error: format!("{e}"),
-                            format: format!("{format:?}"),
-                        })?;
-                    Ok(dctap)
-                }
-            }?;
+        let dctap = match format {
+            DCTAPFormat::CSV => {
+                let dctap = DCTAP::from_path(path, &self.config.tap_config()).map_err(|e| {
+                    RudofError::DCTAPReaderCSV {
+                        path: path_name,
+                        error: format!("{e}"),
+                    }
+                })?;
+                Ok(dctap)
+            } /*DCTAPFormat::XLS | DCTAPFormat::XLSB | DCTAPFormat::XLSM | DCTAPFormat::XLSX => {
+            let path_buf = path.as_ref().to_path_buf();
+            let dctap = DCTAP::from_excel(path_buf, None, &self.config.tap_config())
+            .map_err(|e| RudofError::DCTAPReaderPathXLS {
+            path: path_name,
+            error: format!("{e}"),
+            format: format!("{format:?}"),
+            })?;
+            Ok(dctap)
+            }*/
+            _ => todo!(),
+        }?;
         self.dctap = Some(dctap);
         Ok(())
     }
@@ -450,13 +474,22 @@ impl Rudof {
             }
         }?;
         self.shex_schema = Some(schema_json.clone());
-        let mut schema = CompiledSchema::new();
+        let mut schema = SchemaIR::new();
         schema
             .from_schema_json(&schema_json)
             .map_err(|e| RudofError::CompilingSchemaError {
                 error: format!("{e}"),
             })?;
-        self.shex_validator = Some(ShExValidator::new(schema, &self.config.validator_config()));
+        self.shex_schema_ir = Some(schema.clone());
+
+        let validator =
+            ShExValidator::new(schema, &self.config.validator_config()).map_err(|e| {
+                RudofError::ShExValidatorCreationError {
+                    error: format!("{e}"),
+                    schema: format!("{schema_json}"),
+                }
+            })?;
+        self.shex_validator = Some(validator);
         Ok(())
     }
 
@@ -505,7 +538,7 @@ impl Rudof {
     }
 
     /// Validate RDF data using ShEx
-    /// It uses a ShEx validator which is current ShEx schema and the current ShapeMap that should
+    /// It uses a ShEx validator which has a corrent ShEx schema and the current ShapeMap
     pub fn validate_shex(&mut self) -> Result<ResultShapeMap> {
         let schema_str = format!("{:?}", self.shex_validator);
         match self.shex_validator {
@@ -513,22 +546,21 @@ impl Rudof {
             Some(ref mut validator) => match &self.shapemap {
                 None => Err(RudofError::NoShapeMap { schema: schema_str }),
                 Some(shapemap) => {
-                    validator
-                        .validate_shapemap(shapemap, &self.rdf_data)
+                    let schema = validator.schema().clone();
+                    let result = validator
+                        .validate_shapemap2(
+                            shapemap,
+                            &self.rdf_data,
+                            &schema,
+                            &Some(self.rdf_data.prefixmap_in_memory()),
+                            &None, // TODO!! Get schema prefix map
+                        )
                         .map_err(|e| RudofError::ShExValidatorError {
                             schema: schema_str.clone(),
                             rdf_data: format!("{:?}", self.rdf_data),
                             query_map: format!("{shapemap:?}"),
                             error: format!("{e}"),
                         })?;
-                    let result = &validator
-                        .result_map(Some(self.rdf_data.prefixmap_in_memory()))
-                        .map_err(|e| RudofError::ShExValidatorObtainingResultMapError {
-                            schema: schema_str,
-                            rdf_data: format!("{:?}", self.rdf_data),
-                            shapemap: format!("{shapemap:?}"),
-                            error: format!("{e}"),
-                        })?;
                     Ok(result.clone())
                 }
             },
@@ -667,7 +699,7 @@ impl Rudof {
     }
 }
 
-fn shacl_schema_from_data<RDF: FocusRDF + Debug>(rdf_data: RDF) -> Result<ShaclSchema> {
+fn shacl_schema_from_data<RDF: FocusRDF + Debug>(rdf_data: RDF) -> Result<ShaclSchema<RDF>> {
     let schema = ShaclParser::new(rdf_data)
         .parse()
         .map_err(|e| RudofError::SHACLParseError {
@@ -694,13 +726,48 @@ mod tests {
     use shacl_ast::ShaclFormat;
     use shacl_validation::shacl_processor::ShaclValidationMode;
     use shapemap::ShapeMapFormat;
-    use shex_ast::{compiled::shape_label::ShapeLabel, Node};
+    use shex_ast::{ir::shape_label::ShapeLabel, Node};
     use shex_validation::ShExFormat;
 
     use crate::RudofConfig;
 
     use super::Rudof;
 
+    #[test]
+    fn test_single_shex() {
+        let data = r#"
+        prefix : <http://example/>
+        :x :p 2 .
+        "#;
+        let shex = r#"
+        prefix : <http://example/>
+        :S @:T 
+        :T { :p @:U }
+        :U [ 2 ]
+        "#;
+        let shapemap = r#":x@:S"#;
+        let mut rudof = Rudof::new(&RudofConfig::default());
+        rudof
+            .read_data(
+                data.as_bytes(),
+                &srdf::RDFFormat::Turtle,
+                None,
+                &srdf::ReaderMode::Strict,
+            )
+            .unwrap();
+
+        rudof
+            .read_shex(shex.as_bytes(), &ShExFormat::ShExC, None)
+            .unwrap();
+        rudof
+            .read_shapemap(shapemap.as_bytes(), &ShapeMapFormat::default())
+            .unwrap();
+        let result = rudof.validate_shex().unwrap();
+        let node = Node::iri(iri!("http://example/x"));
+        let shape = ShapeLabel::iri(iri!("http://example/S"));
+        assert!(result.get_info(&node, &shape).unwrap().is_conformant())
+    }
+
     #[test]
     fn test_shex_validation_ok() {
         let data = r#"<http://example/x> <http://example/p> 23 ."#;
