@@ -2,11 +2,16 @@ use std::io;
 
 use iri_s::IriS;
 use shacl_ast::Schema;
+use shacl_ir::compiled_shacl_error::CompiledShaclError;
+use sparql_service::RdfData;
 use srdf::SRDFSparql;
 use thiserror::Error;
 
 #[derive(Error, Debug)]
 pub enum RudofError {
+    #[error("Compiling SHACL: {error}")]
+    ShaclCompilation { error: Box<CompiledShaclError> },
+
     #[error("Error reading config file from path {path}: {error}")]
     RudofConfigFromPathError { path: String, error: io::Error },
 
@@ -37,9 +42,10 @@ pub enum RudofError {
     #[error("ShEx Validator undefined. Before trying to validate with ShEx, a ShEx validator must be initialized in rudof")]
     ShExValidatorUndefined {},
 
-    #[error(
-        "ShEx validation error. Query map: {query_map}\nSchema:\n{schema}\nData:\n{rdf_data}\nError: {error} "
-    )]
+    #[error("Error creating schema for ShEx validation. Schema:\n{schema}\nError: {error} ")]
+    ShExValidatorCreationError { schema: String, error: String },
+
+    #[error("ShEx validation error: {error} ")]
     ShExValidatorError {
         schema: String,
         rdf_data: String,
@@ -91,13 +97,22 @@ pub enum RudofError {
     SHACLParseError { error: String },
 
     #[error("SHACL Compilation from schema {schema} error: {error}")]
-    SHACLCompilationError { error: String, schema: Box<Schema> },
+    SHACLCompilationError {
+        error: String,
+        schema: Box<Schema<RdfData>>,
+    },
 
     #[error("SHACL Validation from schema {schema} error: {error}")]
-    SHACLValidationError { error: String, schema: Box<Schema> },
+    SHACLValidationError {
+        error: String,
+        schema: Box<Schema<RdfData>>,
+    },
 
     #[error("Creating Endpoint validation for SHACL from endpoint {endpoint:?}. error: {error}")]
-    SHACLEndpointValidationCreation { error: String, endpoint: SRDFSparql },
+    SHACLEndpointValidationCreation {
+        error: String,
+        endpoint: Box<SRDFSparql>,
+    },
 
     #[error("Parsing RDF data error: {error}")]
     ParsingRDFDataReader { error: String },
