@@ -1,20 +1,19 @@
 use std::fmt::Display;
 
-use iri_s::iri;
-use srdf::{numeric_literal::NumericLiteral, RDFNode, SHACLPath, SRDFBuilder};
-
-use crate::{
-    component::Component, message_map::MessageMap, severity::Severity, target::Target,
-    SH_DEACTIVATED, SH_DESCRIPTION, SH_GROUP, SH_INFO_STR, SH_NAME, SH_ORDER, SH_PATH,
-    SH_PROPERTY_SHAPE, SH_SEVERITY, SH_VIOLATION_STR, SH_WARNING_STR,
+use crate::shacl_vocab::{
+    sh_deactivated, sh_description, sh_group, sh_info, sh_name, sh_order, sh_path,
+    sh_property_shape, sh_severity, sh_violation, sh_warning,
 };
+use crate::{component::Component, message_map::MessageMap, severity::Severity, target::Target};
+use srdf::Rdf;
+use srdf::{numeric_literal::NumericLiteral, BuildRDF, RDFNode, SHACLPath};
 
-#[derive(Debug, Clone)]
-pub struct PropertyShape {
+#[derive(Debug)]
+pub struct PropertyShape<RDF: Rdf> {
     id: RDFNode,
     path: SHACLPath,
     components: Vec<Component>,
-    targets: Vec<Target>,
+    targets: Vec<Target<RDF>>,
     property_shapes: Vec<RDFNode>,
     closed: bool,
     // ignored_properties: Vec<IriRef>,
@@ -29,7 +28,7 @@ pub struct PropertyShape {
     // annotations: Vec<(IriRef, RDFNode)>,
 }
 
-impl PropertyShape {
+impl<RDF: Rdf> PropertyShape<RDF> {
     pub fn new(id: RDFNode, path: SHACLPath) -> Self {
         PropertyShape {
             id,
@@ -70,7 +69,7 @@ impl PropertyShape {
         self
     }
 
-    pub fn with_targets(mut self, targets: Vec<Target>) -> Self {
+    pub fn with_targets(mut self, targets: Vec<Target<RDF>>) -> Self {
         self.targets = targets;
         self
     }
@@ -127,7 +126,7 @@ impl PropertyShape {
         &self.components
     }
 
-    pub fn targets(&self) -> &Vec<Target> {
+    pub fn targets(&self) -> &Vec<Target<RDF>> {
         &self.targets
     }
 
@@ -177,38 +176,31 @@ impl PropertyShape {
     // }
 
     // TODO: this is a bit ugly
-    pub fn write<RDF>(&self, rdf: &mut RDF) -> Result<(), RDF::Err>
+    pub fn write<B>(&self, rdf: &mut B) -> Result<(), B::Err>
     where
-        RDF: SRDFBuilder,
+        B: BuildRDF,
     {
-        rdf.add_type(&self.id.clone().into(), SH_PROPERTY_SHAPE.clone().into())?;
+        let id: B::Subject = self.id.clone().try_into().map_err(|_| unreachable!())?;
+        rdf.add_type(id.clone(), sh_property_shape().clone())?;
 
         self.name.iter().try_for_each(|(lang, value)| {
-            let literal: RDF::Literal = match lang {
+            let literal: B::Literal = match lang {
                 Some(_) => todo!(),
                 None => value.clone().into(),
             };
-            rdf.add_triple(
-                &self.id.clone().try_into().map_err(|_| unreachable!())?,
-                &SH_NAME.clone().into(),
-                &literal.into(),
-            )
+            rdf.add_triple(id.clone(), sh_name().clone(), literal)
         })?;
 
         self.description.iter().try_for_each(|(lang, value)| {
-            let literal: RDF::Literal = match lang {
+            let literal: B::Literal = match lang {
                 Some(_) => todo!(),
                 None => value.clone().into(),
             };
-            rdf.add_triple(
-                &self.id.clone().try_into().map_err(|_| unreachable!())?,
-                &SH_DESCRIPTION.clone().into(),
-                &literal.into(),
-            )
+            rdf.add_triple(id.clone(), sh_description().clone(), literal)
         })?;
 
         if let Some(order) = self.order.clone() {
-            let literal: RDF::Literal = match order {
+            let literal: B::Literal = match order {
                 NumericLiteral::Decimal(_) => todo!(),
                 NumericLiteral::Double(float) => float.into(),
                 NumericLiteral::Integer(int) => {
@@ -216,27 +208,15 @@ impl PropertyShape {
                     i.into()
                 }
             };
-            rdf.add_triple(
-                &self.id.clone().try_into().map_err(|_| unreachable!())?,
-                &SH_ORDER.clone().into(),
-                &literal.into(),
-            )?;
+            rdf.add_triple(id.clone(), sh_order().clone(), literal)?;
         }
 
         if let Some(group) = &self.group {
-            rdf.add_triple(
-                &self.id.clone().try_into().map_err(|_| unreachable!())?,
-                &SH_GROUP.clone().into(),
-                &group.clone().into(),
-            )?;
+            rdf.add_triple(id.clone(), sh_group().clone(), group.clone())?;
         }
 
         if let SHACLPath::Predicate { pred } = &self.path {
-            rdf.add_triple(
-                &self.id.clone().try_into().map_err(|_| unreachable!())?,
-                &SH_PATH.clone().into(),
-                &pred.clone().into(),
-            )?;
+            rdf.add_triple(id.clone(), sh_path().clone(), pred.clone())?;
         } else {
             unimplemented!()
         }
@@ -250,35 +230,27 @@ impl PropertyShape {
             .try_for_each(|target| target.write(&self.id, rdf))?;
 
         if self.deactivated {
-            let literal: RDF::Literal = "true".to_string().into();
+            let literal: B::Literal = "true".to_string().into();
 
-            rdf.add_triple(
-                &self.id.clone().try_into().map_err(|_| unreachable!())?,
-                &SH_DEACTIVATED.clone().into(),
-                &literal.into(),
-            )?;
+            rdf.add_triple(id.clone(), sh_deactivated().clone(), literal)?;
         }
 
         if let Some(severity) = &self.severity {
             let pred = match severity {
-                Severity::Violation => iri!(SH_VIOLATION_STR),
-                Severity::Info => iri!(SH_INFO_STR),
-                Severity::Warning => iri!(SH_WARNING_STR),
-                Severity::Generic(iri) => iri.get_iri().unwrap(),
+                Severity::Violation => sh_violation(),
+                Severity::Info => sh_info(),
+                Severity::Warning => sh_warning(),
+                Severity::Generic(iri) => &iri.get_iri().unwrap(),
             };
 
-            rdf.add_triple(
-                &self.id.clone().try_into().map_err(|_| unreachable!())?,
-                &SH_SEVERITY.clone().into(),
-                &pred.clone().into(),
-            )?;
+            rdf.add_triple(id.clone(), sh_severity().clone(), pred.clone())?;
         }
 
         Ok(())
     }
 }
 
-impl Display for PropertyShape {
+impl<RDF: Rdf> Display for PropertyShape<RDF> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         writeln!(f, "{{")?;
         writeln!(f, "       PropertyShape")?;
@@ -299,3 +271,39 @@ impl Display for PropertyShape {
         Ok(())
     }
 }
+
+impl<RDF: Rdf> Clone for PropertyShape<RDF> {
+    fn clone(&self) -> Self {
+        Self {
+            id: self.id.clone(),
+            path: self.path.clone(),
+            components: self.components.clone(),
+            targets: self.targets.clone(),
+            property_shapes: self.property_shapes.clone(),
+            closed: self.closed,
+            deactivated: self.deactivated,
+            severity: self.severity.clone(),
+            name: self.name.clone(),
+            description: self.description.clone(),
+            order: self.order.clone(),
+            group: self.group.clone(),
+        }
+    }
+}
+
+impl<RDF: Rdf> PartialEq for PropertyShape<RDF> {
+    fn eq(&self, other: &Self) -> bool {
+        self.id == other.id
+            && self.path == other.path
+            && self.components == other.components
+            && self.targets == other.targets
+            && self.property_shapes == other.property_shapes
+            && self.closed == other.closed
+            && self.deactivated == other.deactivated
+            && self.severity == other.severity
+            && self.name == other.name
+            && self.description == other.description
+            && self.order == other.order
+            && self.group == other.group
+    }
+}
