@@ -2,25 +2,43 @@ use std::fmt::Display;
 
 use prefixmap::PrefixMapError;
 use rbe::RbeError;
-use shex_ast::compiled::preds::Preds;
-use shex_ast::compiled::shape_expr::ShapeExpr;
-use shex_ast::{compiled::shape_label::ShapeLabel, Node, Pred, ShapeExprLabel, ShapeLabelIdx};
+use shex_ast::ir::preds::Preds;
+use shex_ast::ir::shape::Shape;
+use shex_ast::ir::shape_expr::ShapeExpr;
+use shex_ast::{ir::shape_label::ShapeLabel, Node, Pred, ShapeExprLabel, ShapeLabelIdx};
 use srdf::Object;
 use thiserror::Error;
 
+use crate::Reasons;
+
 #[derive(Error, Debug, Clone)]
 pub enum ValidatorError {
+    #[error("Converting Term to RDFNode failed pending {term}")]
+    TermToRDFNodeFailed { term: String },
+
+    #[error("Failed pending: RBE passed, but pending references failed")]
+    FailedPending {
+        failed_pending: Vec<(Node, ShapeLabelIdx)>,
+    },
+    #[error("Negation cycle error: {neg_cycles:?}")]
+    NegCycleError {
+        neg_cycles: Vec<Vec<(String, String, Vec<String>)>>,
+    },
+
     #[error("SRDF Error: {error}")]
     SRDFError { error: String },
 
     #[error("Not found shape label {shape}")]
     NotFoundShapeLabel { shape: ShapeLabel },
 
+    #[error("Not found shape label with index {idx}")]
+    NotFoundShapeLabelWithIndex { idx: ShapeLabelIdx },
+
     #[error("Error converting object to iri: {object}")]
     ConversionObjectIri { object: Object },
 
     #[error("Compiling schema: {error}")]
-    CompiledSchemaError { error: String },
+    SchemaIRError { error: String },
 
     #[error("Shapemap error: {error}")]
     ShapeMapError { error: String },
@@ -45,11 +63,27 @@ pub enum ValidatorError {
 
     #[error("And error: shape expression {shape_expr} failed for node {node}: {errors}")]
     ShapeAndError {
-        shape_expr: ShapeExpr,
-        node: Node,
+        shape_expr: Box<ShapeExpr>,
+        node: Box<Node>,
         errors: ValidatorErrors,
     },
 
+    #[error("OR error: shape expression {shape_expr} failed for node {node}: all branches failed")]
+    ShapeOrError {
+        shape_expr: Box<ShapeExpr>,
+        node: Box<Node>,
+        errors: Vec<(ShapeExpr, ValidatorErrors)>,
+    },
+
+    #[error(
+        "Shape Not error: failed for node {node} because it passed {shape_expr} with {reasons}"
+    )]
+    ShapeNotError {
+        shape_expr: Box<ShapeExpr>,
+        node: Box<Node>,
+        reasons: Reasons,
+    },
+
     #[error("Error reading config file from path {path}: {error}")]
     ValidatorConfigFromPathError { path: String, error: String },
 
@@ -76,6 +110,19 @@ pub enum ValidatorError {
         label: String,
         error: String,
     },
+
+    #[error("Shape not found for index {idx}")]
+    ShapeExprNotFound { idx: ShapeLabelIdx },
+
+    #[error("Shape fails for node {node} with shape {shape}")]
+    ShapeFails {
+        node: Box<Node>,
+        shape: Box<Shape>,
+        errors: Vec<ValidatorError>,
+    },
+
+    #[error("ShapeRef fails for node {node} with idx: {idx}")]
+    ShapeRefFailed { node: Box<Node>, idx: ShapeLabelIdx },
 }
 
 #[derive(Debug, Clone)]
