@@ -11,17 +11,17 @@ use oxrdf::{
 use oxrdfio::RdfFormat;
 use prefixmap::PrefixMap;
 use sparesults::QuerySolution as SparQuerySolution;
+use srdf::BuildRDF;
 use srdf::FocusRDF;
-use srdf::Query;
+use srdf::NeighsRDF;
+use srdf::QueryRDF;
 use srdf::QuerySolution;
 use srdf::QuerySolutions;
 use srdf::RDFFormat;
 use srdf::Rdf;
 use srdf::ReaderMode;
-use srdf::SRDFBuilder;
 use srdf::SRDFGraph;
 use srdf::SRDFSparql;
-use srdf::Sparql;
 use srdf::VarName;
 use srdf::RDF_TYPE_STR;
 use std::fmt::Debug;
@@ -106,6 +106,17 @@ impl RdfData {
         self.graph = None
     }
 
+    pub fn from_str(
+        data: &str,
+        format: &RDFFormat,
+        base: Option<&str>,
+        reader_mode: &ReaderMode,
+    ) -> Result<RdfData, RdfDataError> {
+        let mut rdf_data = Self::new();
+        rdf_data.merge_from_reader(data.as_bytes(), format, base, reader_mode)?;
+        Ok(rdf_data)
+    }
+
     /// Merge the in-memory graph with the graph read from a reader
     pub fn merge_from_reader<R: io::Read>(
         &mut self,
@@ -154,12 +165,12 @@ impl RdfData {
     }
 
     pub fn show_blanknode(&self, bn: &OxBlankNode) -> String {
-        let str: String = format!("{}", bn);
+        let str: String = format!("{bn}");
         format!("{}", str.green())
     }
 
     pub fn show_literal(&self, lit: &OxLiteral) -> String {
-        let str: String = format!("{}", lit);
+        let str: String = format!("{lit}");
         format!("{}", str.red())
     }
 
@@ -257,7 +268,7 @@ impl Rdf for RdfData {
     }
 }
 
-impl Sparql for RdfData {
+impl QueryRDF for RdfData {
     fn query_select(&self, query_str: &str) -> Result<QuerySolutions<RdfData>, RdfDataError>
     where
         Self: Sized,
@@ -329,10 +340,10 @@ fn _rdf_type() -> OxNamedNode {
     OxNamedNode::new_unchecked(RDF_TYPE_STR)
 }
 
-impl Query for RdfData {
+impl NeighsRDF for RdfData {
     fn triples(&self) -> Result<impl Iterator<Item = Self::Triple>, Self::Err> {
-        let endpoints_triples = self.endpoints.iter().flat_map(Query::triples).flatten();
-        let graph_triples = self.graph.iter().flat_map(Query::triples).flatten();
+        let endpoints_triples = self.endpoints.iter().flat_map(NeighsRDF::triples).flatten();
+        let graph_triples = self.graph.iter().flat_map(NeighsRDF::triples).flatten();
         Ok(endpoints_triples.chain(graph_triples))
     }
 }
@@ -347,7 +358,7 @@ impl FocusRDF for RdfData {
     }
 }
 
-impl SRDFBuilder for RdfData {
+impl BuildRDF for RdfData {
     fn empty() -> Self {
         todo!()
     }
@@ -364,25 +375,29 @@ impl SRDFBuilder for RdfData {
         todo!()
     }
 
-    fn add_triple(
-        &mut self,
-        _subj: &Self::Subject,
-        _pred: &Self::IRI,
-        _obj: &Self::Term,
-    ) -> Result<(), Self::Err> {
+    fn add_triple<S, P, O>(&mut self, _subj: S, _pred: P, _obj: O) -> Result<(), Self::Err>
+    where
+        S: Into<Self::Subject>,
+        P: Into<Self::IRI>,
+        O: Into<Self::Term>,
+    {
         todo!()
     }
 
-    fn remove_triple(
-        &mut self,
-        _subj: &Self::Subject,
-        _pred: &Self::IRI,
-        _obj: &Self::Term,
-    ) -> Result<(), Self::Err> {
+    fn remove_triple<S, P, O>(&mut self, _subj: S, _pred: P, _obj: O) -> Result<(), Self::Err>
+    where
+        S: Into<Self::Subject>,
+        P: Into<Self::IRI>,
+        O: Into<Self::Term>,
+    {
         todo!()
     }
 
-    fn add_type(&mut self, _node: &Self::Term, _type_: Self::Term) -> Result<(), Self::Err> {
+    fn add_type<S, T>(&mut self, _node: S, _type_: T) -> Result<(), Self::Err>
+    where
+        S: Into<Self::Subject>,
+        T: Into<Self::Term>,
+    {
         todo!()
     }
 
@@ -402,4 +417,14 @@ impl SRDFBuilder for RdfData {
         }
         Ok(())
     }
+
+    fn add_bnode(&mut self) -> Result<Self::BNode, Self::Err> {
+        match self.graph {
+            Some(ref mut graph) => {
+                let bnode = graph.add_bnode()?;
+                Ok(bnode)
+            }
+            None => Err(RdfDataError::BNodeNoGraph),
+        }
+    }
 }
