@@ -1,5 +1,5 @@
 use crate::async_srdf::AsyncSRDF;
-use crate::{FocusRDF, Query, RDFFormat, Rdf, SRDFBuilder, RDF_TYPE_STR};
+use crate::{BuildRDF, FocusRDF, NeighsRDF, RDFFormat, Rdf, RDF_TYPE_STR};
 use async_trait::async_trait;
 use colored::*;
 use iri_s::IriS;
@@ -15,7 +15,8 @@ use tracing::debug;
 use crate::srdfgraph_error::SRDFGraphError;
 use oxrdf::{
     BlankNode as OxBlankNode, Graph, GraphName, Literal as OxLiteral, NamedNode as OxNamedNode,
-    Quad, Subject as OxSubject, Term as OxTerm, Triple as OxTriple, TripleRef,
+    NamedNodeRef, Quad, Subject as OxSubject, SubjectRef, Term as OxTerm, TermRef,
+    Triple as OxTriple, TripleRef,
 };
 use oxttl::{NQuadsParser, NTriplesParser, TurtleParser};
 use prefixmap::{prefixmap::*, PrefixMapError};
@@ -26,6 +27,7 @@ pub struct SRDFGraph {
     graph: Graph,
     pm: PrefixMap,
     base: Option<IriS>,
+    bnode_counter: usize,
 }
 
 impl SRDFGraph {
@@ -153,12 +155,12 @@ impl SRDFGraph {
     }
 
     pub fn show_blanknode(&self, bn: &OxBlankNode) -> String {
-        let str: String = format!("{}", bn);
+        let str: String = format!("{bn}");
         format!("{}", str.green())
     }
 
     pub fn show_literal(&self, lit: &OxLiteral) -> String {
-        let str: String = format!("{}", lit);
+        let str: String = format!("{lit}");
         format!("{}", str.red())
     }
 
@@ -175,6 +177,25 @@ impl SRDFGraph {
         OxNamedNode::new_unchecked(iri.as_str())
     }
 
+    pub fn add_triple_ref<'a, S, P, O>(
+        &mut self,
+        subj: S,
+        pred: P,
+        obj: O,
+    ) -> Result<(), SRDFGraphError>
+    where
+        S: Into<SubjectRef<'a>>,
+        P: Into<NamedNodeRef<'a>>,
+        O: Into<TermRef<'a>>,
+    {
+        let subj: SubjectRef<'a> = subj.into();
+        let pred: NamedNodeRef<'a> = pred.into();
+        let obj: TermRef<'a> = obj.into();
+        let triple = TripleRef::new(subj, pred, obj);
+        self.graph.insert(triple);
+        Ok(())
+    }
+
     pub fn merge_from_path<P: AsRef<Path>>(
         &mut self,
         path: P,
@@ -269,7 +290,7 @@ impl Rdf for SRDFGraph {
     }
 }
 
-impl Query for SRDFGraph {
+impl NeighsRDF for SRDFGraph {
     fn triples(&self) -> Result<impl Iterator<Item = Self::Triple>, Self::Err> {
         Ok(self.graph.iter().map(TripleRef::into_owned))
     }
@@ -339,7 +360,7 @@ impl FocusRDF for SRDFGraph {
     }
 }
 
-impl SRDFBuilder for SRDFGraph {
+impl BuildRDF for SRDFGraph {
     fn add_base(&mut self, base: &Option<IriS>) -> Result<(), Self::Err> {
         self.base.clone_from(base);
         Ok(())
@@ -355,36 +376,46 @@ impl SRDFBuilder for SRDFGraph {
         Ok(())
     }
 
-    fn add_triple(
-        &mut self,
-        subj: &Self::Subject,
-        pred: &Self::IRI,
-        obj: &Self::Term,
-    ) -> Result<(), Self::Err> {
-        let triple = OxTriple::new(subj.clone(), pred.clone(), obj.clone());
+    fn add_bnode(&mut self) -> Result<Self::BNode, Self::Err> {
+        self.bnode_counter += 1;
+        match self.bnode_counter.try_into() {
+            Ok(bn) => Ok(OxBlankNode::new_from_unique_id(bn)),
+            Err(_) => Err(SRDFGraphError::BlankNodeId {
+                msg: format!("Error converting {} to usize", self.bnode_counter),
+            }),
+        }
+    }
+
+    fn add_triple<S, P, O>(&mut self, subj: S, pred: P, obj: O) -> Result<(), Self::Err>
+    where
+        S: Into<Self::Subject>,
+        P: Into<Self::IRI>,
+        O: Into<Self::Term>,
+    {
+        let triple = OxTriple::new(subj.into(), pred.into(), obj.into());
         self.graph.insert(&triple);
         Ok(())
     }
 
-    fn remove_triple(
-        &mut self,
-        subj: &Self::Subject,
-        pred: &Self::IRI,
-        obj: &Self::Term,
-    ) -> Result<(), Self::Err> {
-        let triple = OxTriple::new(subj.clone(), pred.clone(), obj.clone());
+    fn remove_triple<S, P, O>(&mut self, subj: S, pred: P, obj: O) -> Result<(), Self::Err>
+    where
+        S: Into<Self::Subject>,
+        P: Into<Self::IRI>,
+        O: Into<Self::Term>,
+    {
+        let triple = OxTriple::new(subj.into(), pred.into(), obj.into());
         self.graph.remove(&triple);
         Ok(())
     }
 
-    fn add_type(&mut self, node: &Self::Term, r#type: Self::Term) -> Result<(), Self::Err> {
-        let subject: Self::Subject =
-            node.clone()
-                .try_into()
-                .map_err(|_| SRDFGraphError::UnexepectedNodeType {
-                    node: node.to_string(),
-                })?;
-        let triple = OxTriple::new(subject, rdf_type(), r#type.clone());
+    fn add_type<S, T>(&mut self, node: S, type_: T) -> Result<(), Self::Err>
+    where
+        S: Into<Self::Subject>,
+        T: Into<Self::Term>,
+    {
+        let subject: Self::Subject = node.into();
+        let type_: Self::Term = type_.into();
+        let triple = OxTriple::new(subject, rdf_type(), type_.clone());
         self.graph.insert(&triple);
         Ok(())
     }
@@ -395,6 +426,7 @@ impl SRDFBuilder for SRDFGraph {
             graph: Graph::new(),
             pm: PrefixMap::new(),
             base: None,
+            bnode_counter: 0,
         }
     }
 
@@ -455,13 +487,13 @@ impl ReaderMode {
 
 #[cfg(test)]
 mod tests {
-    use std::collections::HashSet;
-
+    use crate::neighs_rdf::NeighsRDF;
     use iri_s::IriS;
     use oxrdf::Literal as OxLiteral;
     use oxrdf::NamedNode as OxNamedNode;
     use oxrdf::Subject as OxSubject;
     use oxrdf::Term as OxTerm;
+    use std::collections::HashSet;
 
     use crate::iri;
     use crate::matcher::Any;
@@ -477,11 +509,11 @@ mod tests {
     use crate::satisfy;
     use crate::set_focus;
     use crate::PResult;
-    use crate::Query as _;
+    // use crate::Query as _;
+    use crate::BuildRDF;
     use crate::RDFFormat;
     use crate::RDFNodeParse as _;
     use crate::RDFParseError;
-    use crate::SRDFBuilder;
     use crate::Triple;
 
     use super::ReaderMode;
@@ -662,7 +694,7 @@ mod tests {
         let knows = OxNamedNode::new_unchecked("http://example.org/knows");
         let bob = OxTerm::NamedNode(OxNamedNode::new_unchecked("http://example.org/bob"));
 
-        graph.add_triple(&alice, &knows, &bob).unwrap();
+        graph.add_triple(alice, knows, bob).unwrap();
 
         assert_eq!(graph.len(), 1);
     }
@@ -847,7 +879,18 @@ mod tests {
     #[test]
     fn test_iri() {
         let graph = SRDFGraph::default();
-        let x = IriS::from_named_node(&OxNamedNode::new_unchecked("http://example.org/x"));
-        assert_eq!(iri().parse(&x, graph).unwrap(), x)
+        let x = OxNamedNode::new_unchecked("http://example.org/x");
+        let x_iri = IriS::from_named_node(&x);
+        assert_eq!(iri().parse(&x_iri, graph).unwrap(), x)
+    }
+
+    #[test]
+    fn test_add_triple_ref() {
+        let mut graph = SRDFGraph::default();
+        let s = OxNamedNode::new_unchecked("http://example.org/x");
+        let p = OxNamedNode::new_unchecked("http://example.org/p");
+        let o = OxNamedNode::new_unchecked("http://example.org/y");
+        graph.add_triple_ref(&s, &p, &o).unwrap();
+        assert_eq!(graph.len(), 1);
     }
 }
