@@ -11,6 +11,9 @@ pub enum SRDFGraphError {
     #[error("Parsing base iri {str}: error: {error}")]
     BaseParseError { str: String, error: String },
 
+    #[error("Blank node generation id: {msg}")]
+    BlankNodeId { msg: String },
+
     #[error("Reading path {path_name:?} error: {error:?}")]
     ReadingPathError { path_name: String, error: io::Error },
 
